pub enum RelationshipAccessorInitializer {
Relationship {
entity_field_offset: usize,
linked_spawn: bool,
allow_self_referential: bool,
relationship_target_getter: Arc<dyn Fn(&Components) -> Option<ComponentId>>,
},
RelationshipTarget {
iter: for<'a> unsafe fn(Ptr<'a>) -> Box<dyn Iterator<Item = Entity> + 'a>,
linked_spawn: bool,
allow_self_referential: bool,
relationship_getter: Arc<dyn Fn(&Components) -> Option<ComponentId>>,
},
}Expand description
Initializer enum for RelationshipAccessor that allows to configure relationship for dynamic components.
Variants
Relationship
Describes a Relationship component.
Fields
entity_field_offset: usizeOffset of the field containing Entity from the base of the component.
Dynamic equivalent of Relationship::get.
linked_spawn: boolValue of RelationshipTarget::LINKED_SPAWN for the Relationship::RelationshipTarget of this Relationship.
allow_self_referential: boolValue of Relationship::ALLOW_SELF_REFERENTIAL of this Relationship.
relationship_target_getter: Arc<dyn Fn(&Components) -> Option<ComponentId>>Getter for ComponentId of the RelationshipTarget counterpart.
Should return None if RelationshipTarget isn’t registered yet.
RelationshipTarget
Describes a RelationshipTarget component.
Fields
iter: for<'a> unsafe fn(Ptr<'a>) -> Box<dyn Iterator<Item = Entity> + 'a>Function that returns an iterator over all Entitys of this RelationshipTarget’s collection.
Dynamic equivalent of RelationshipTarget::iter.
Safety
Passed pointer must point to the value of the same component as the one that this accessor was registered to.
linked_spawn: boolValue of RelationshipTarget::LINKED_SPAWN of this RelationshipTarget.
allow_self_referential: boolValue of Relationship::ALLOW_SELF_REFERENTIAL for the Relationship of this RelationshipTarget.
relationship_getter: Arc<dyn Fn(&Components) -> Option<ComponentId>>Getter for ComponentId of the Relationship counterpart.
Should return None if Relationship isn’t registered yet.